import unittest
from enum import Enum

from robot.running.arguments.argumentspec import ArgumentSpec, ArgInfo
from robot.utils.asserts import assert_equal


class TestStringRepr(unittest.TestCase):

    def test_empty(self):
        self._verify('')

    def test_normal(self):
        self._verify('a, b', ['a', 'b'])

    def test_non_ascii_names(self):
        self._verify('nön, äscii', ['nön', 'äscii'])

    def test_default(self):
        self._verify('a, b=c', ['a', 'b'], defaults={'b': 'c'})
        self._verify('nön=äscii', ['nön'], defaults={'nön': 'äscii'})
        self._verify('i=42', ['i'], defaults={'i': 42})

    def test_default_as_bytes(self):
        self._verify('b=ytes', ['b'], defaults={'b': b'ytes'})
        self._verify('ä=\\xe4', ['ä'], defaults={'ä': b'\xe4'})

    def test_type_as_class(self):
        self._verify('a: int, b: bool', ['a', 'b'], types={'a': int, 'b': bool})

    def test_type_as_string(self):
        self._verify('a: Integer, b: Boolean', ['a', 'b'],
                     types={'a': 'Integer', 'b': 'Boolean'})

    def test_type_and_default(self):
        self._verify('arg: int = 1', ['arg'], types=[int], defaults={'arg': 1})

    def test_positional_only(self):
        self._verify('a, /', positional_only=['a'])
        self._verify('a, /, b', positional_only=['a'], positional_or_named=['b'])

    def test_positional_only_with_default(self):
        self._verify('a, b=2, /', positional_only=['a', 'b'], defaults={'b': 2})

    def test_positional_only_with_type(self):
        self._verify('a: int, b, /', positional_only=['a', 'b'], types=[int])
        self._verify('a: int, b: float, /, c: bool, d',
                     positional_only=['a', 'b'],
                     positional_or_named=['c', 'd'],
                     types=[int, float, bool])

    def test_positional_only_with_type_and_default(self):
        self._verify('a: int = 1, b=2, /',
                     positional_only=['a', 'b'],
                     types={'a': int},
                     defaults={'a': 1, 'b': 2})

    def test_varargs(self):
        self._verify('*varargs',
                     var_positional='varargs')
        self._verify('a, *b',
                     positional_or_named=['a'],
                     var_positional='b')

    def test_varargs_with_type(self):
        self._verify('*varargs: float',
                     var_positional='varargs',
                     types={'varargs': float})
        self._verify('a: int, *b: list[int]',
                     positional_or_named=['a'],
                     var_positional='b',
                     types=[int, 'list[int]'])

    def test_named_only_without_varargs(self):
        self._verify('*, kwo',
                     named_only=['kwo'])

    def test_named_only_with_varargs(self):
        self._verify('*varargs, k1, k2',
                     var_positional='varargs',
                     named_only=['k1', 'k2'])

    def test_named_only_with_default(self):
        self._verify('*, k=1, w, o=3',
                     named_only=['k', 'w', 'o'],
                     defaults={'k': 1, 'o': 3})

    def test_named_only_with_types(self):
        self._verify('*, k: int, w: float, o',
                     named_only=['k', 'w', 'o'],
                     types=[int, float])
        self._verify('x: int, *y: float, z: bool',
                     positional_or_named=['x'],
                     var_positional='y',
                     named_only=['z'],
                     types=[int, float, bool])

    def test_named_only_with_types_and_defaults(self):
        self._verify('x: int = 1, *, y: float, z: bool = 3',
                     positional_or_named=['x'],
                     named_only=['y', 'z'],
                     types=[int, float, bool],
                     defaults={'x': 1, 'z': 3})

    def test_kwargs(self):
        self._verify('**kws',
                     var_named='kws')
        self._verify('a, b=c, *d, e=f, g, **h',
                     positional_or_named=['a', 'b'],
                     var_positional='d',
                     named_only=['e', 'g'],
                     var_named='h',
                     defaults={'b': 'c', 'e': 'f'})

    def test_kwargs_with_types(self):
        self._verify('**kws: dict[str, int]',
                     var_named='kws',
                     types={'kws': 'dict[str, int]'})
        self._verify('a: int, /, b: float, *c: list[int], d: bool, **e: dict[int, str]',
                     positional_only=['a'],
                     positional_or_named=['b'],
                     var_positional='c',
                     named_only=['d'],
                     var_named='e',
                     types=[int, float, 'list[int]', bool, 'dict[int, str]'])

    def test_enum_with_few_members(self):
        class Small(Enum):
            ONLY_FEW_MEMBERS = 1
            SO_THEY_CAN = 2
            BE_PRETTY_LONG = 3
        self._verify('e: Small',
                     ['e'], types=[Small])

    def test_enum_with_many_short_members(self):
        class ManyShort(Enum):
            ONE = 1
            TWO = 2
            THREE = 3
            FOUR = 4
            FIVE = 5
            SIX = 6
        self._verify('e: ManyShort',
                     ['e'], types=[ManyShort])

    def test_enum_with_many_long_members(self):
        class Big(Enum):
            MANY_MEMBERS = 1
            THAT_ARE_LONGISH = 2
            MEANS_THEY_ALL_DO_NOT_FIT = 3
            AND_SOME_ARE_OMITTED = 4
            FROM_THE_END = 5
        self._verify('e: Big',
                     ['e'], types=[Big])

    def _verify(self, expected, positional_or_named=None, **config):
        spec = ArgumentSpec(positional_or_named=positional_or_named, **config)
        assert_equal(str(spec), expected)
        assert_equal(bool(spec), bool(expected))


class TestArgInfo(unittest.TestCase):

    def test_required_without_default(self):
        for kind in (ArgInfo.POSITIONAL_ONLY,
                     ArgInfo.POSITIONAL_OR_NAMED,
                     ArgInfo.NAMED_ONLY):
            assert_equal(ArgInfo(kind).required, True)
            assert_equal(ArgInfo(kind, default=None).required, False)

    def test_never_required(self):
        for kind in (ArgInfo.VAR_POSITIONAL,
                     ArgInfo.VAR_NAMED,
                     ArgInfo.POSITIONAL_ONLY_MARKER,
                     ArgInfo.NAMED_ONLY_MARKER):
            assert_equal(ArgInfo(kind).required, False)


if __name__ == '__main__':
    unittest.main()



import unittest

from robot.utils.asserts import assert_equal
from robot.utils.escaping import escape, unescape, split_from_equals


def assert_unescape(inp, exp):
    assert_equal(unescape(inp), exp, repr(inp))


class TestUnEscape(unittest.TestCase):

    def test_no_backslash(self):
        for inp in ['no escapes', '', 42]:
            assert_unescape(inp, inp)

    def test_single_backslash(self):
        for inp, exp in [('\\', ''),
                         ('\\ ', ' '),
                         ('\\ ', ' '),
                         ('a\\', 'a'),
                         ('\\a', 'a'),
                         ('\\-', u'-'),
                         (u'\\\xe4', u'\xe4'),
                         ('\\0', '0'),
                         ('a\\b\\c\\d', 'abcd')]:
            assert_unescape(inp, exp)

    def test_multiple_backslash(self):
        for inp, exp in [('\\\\', '\\'),
                         ('\\\\\\', '\\'),
                         ('\\\\\\\\', '\\\\'),
                         ('\\\\\\\\\\', '\\\\'),
                         ('x\\\\x', 'x\\x'),
                         ('x\\\\\\x', 'x\\x'),
                         ('x\\\\\\\\x', 'x\\\\x')]:
            assert_unescape(inp, exp)

    def test_newline(self):
        for inp, exp in [('\\n', '\n'),
                         ('\\\\n', '\\n'),
                         ('\\\\\\n', '\\\n'),
                         ('\\n ', '\n '),
                         ('\\\\n ', '\\n '),
                         ('\\\\\\n ', '\\\n '),
                         ('\\nx', '\nx'),
                         ('\\\\nx', '\\nx'),
                         ('\\\\\\nx', '\\\nx'),
                         ('\\n x', '\n x'),
                         ('\\\\n x', '\\n x'),
                         ('\\\\\\n x', '\\\n x')]:
            assert_unescape(inp, exp)

    def test_carriage_return(self):
        for inp, exp in [('\\r', '\r'),
                         ('\\\\r', '\\r'),
                         ('\\\\\\r', '\\\r'),
                         ('\\r ', '\r '),
                         ('\\\\r ', '\\r '),
                         ('\\\\\\r ', '\\\r '),
                         ('\\rx', '\rx'),
                         ('\\\\rx', '\\rx'),
                         ('\\\\\\rx', '\\\rx'),
                         ('\\r x', '\r x'),
                         ('\\\\r x', '\\r x'),
                         ('\\\\\\r x', '\\\r x')]:
            assert_unescape(inp, exp)

    def test_tab(self):
        for inp, exp in [('\\t', '\t'),
                         ('\\\\t', '\\t'),
                         ('\\\\\\t', '\\\t'),
                         ('\\t ', '\t '),
                         ('\\\\t ', '\\t '),
                         ('\\\\\\t ', '\\\t '),
                         ('\\tx', '\tx'),
                         ('\\\\tx', '\\tx'),
                         ('\\\\\\tx', '\\\tx'),
                         ('\\t x', '\t x'),
                         ('\\\\t x', '\\t x'),
                         ('\\\\\\t x', '\\\t x')]:
            assert_unescape(inp, exp)

    def test_invalid_x(self):
        for inp in r'\x \xxx xx\xxx \x0 \x0g \X00 \x-1 \x+1'.split():
            assert_unescape(inp, inp.replace('\\', ''))

    def test_valid_x(self):
        for inp, exp in [(r'\x00', u'\x00'),
                         (r'\xab\xBA', u'\xab\xba'),
                         (r'\xe4iti', u'\xe4iti')]:
            assert_unescape(inp, exp)

    def test_invalid_u(self):
        for inp in r'''\u
                       \ukekkonen
                       b\uu
                       \u0
                       \u123
                       \u123x
                       \u-123
                       \u+123
                       \u1.23'''.split():
            assert_unescape(inp, inp.replace('\\', ''))

    def test_valid_u(self):
        for inp, exp in [(r'\u0000', u'\x00'),
                         (r'\uABba', u'\uabba'),
                         (r'\u00e4iti', u'\xe4iti')]:
            assert_unescape(inp, exp)

    def test_invalid_U(self):
        for inp in r'''\U
                       \Ukekkonen
                       b\Uu
                       \U0
                       \U1234567
                       \U1234567x
                       \U-1234567
                       \U+1234567
                       \U1.234567'''.split():
            assert_unescape(inp, inp.replace('\\', ''))

    def test_valid_U(self):
        for inp, exp in [(r'\U00000000', u'\x00'),
                         (r'\U0000ABba', u'\uabba'),
                         (r'\U0001f3e9', u'\U0001f3e9'),
                         (r'\U0010FFFF', u'\U0010ffff'),
                         (r'\U000000e4iti', u'\xe4iti')]:
            assert_unescape(inp, exp)

    def test_U_above_valid_range(self):
        assert_unescape(r'\U00110000', 'U00110000')
        assert_unescape(r'\U12345678', 'U12345678')
        assert_unescape(r'\UffffFFFF', 'UffffFFFF')


class TestEscape(unittest.TestCase):

    def test_escape(self):
        for inp, exp in [('nothing to escape', 'nothing to escape'),
                         ('still nothing $ @', 'still nothing $ @' ),
                         ('1 backslash to 2: \\', '1 backslash to 2: \\\\'),
                         ('3 bs to 6: \\\\\\', '3 bs to 6: \\\\\\\\\\\\'),
                         ('\\' * 1000, '\\' * 2000 ),
                         ('${notvar}', '\\${notvar}'),
                         ('@{notvar}', '\\@{notvar}'),
                         ('${nv} ${nv} @{nv}', '\\${nv} \\${nv} \\@{nv}'),
                         ('\\${already esc}', '\\\\\\${already esc}'),
                         ('\\${ae} \\\\@{ae} \\\\\\@{ae}',
                          '\\\\\\${ae} \\\\\\\\\\@{ae} \\\\\\\\\\\\\\@{ae}'),
                         ('%{reserved}', '\\%{reserved}'),
                         ('&{reserved}', '\\&{reserved}'),
                         ('*{reserved}', '\\*{reserved}'),
                         ('x{notreserved}', 'x{notreserved}'),
                         ('named=arg', 'named\\=arg')]:
            assert_equal(escape(inp), exp, inp)

    def test_escape_control_words(self):
        for inp in ['ELSE', 'ELSE IF', 'AND', 'WITH NAME', 'AS']:
            assert_equal(escape(inp), '\\' + inp)
            assert_equal(escape(inp.lower()), inp.lower())
            assert_equal(escape('other' + inp), 'other' + inp)
            assert_equal(escape(inp + ' '), inp + ' ')


class TestSplitFromEquals(unittest.TestCase):

    def test_basics(self):
        for inp in 'foo=bar', '=', 'split=from=first', '===':
            self._test(inp, *inp.split('=', 1))

    def test_escaped(self):
        self._test(r'a\=b=c', r'a\=b', 'c')
        self._test(r'\=====', r'\=', '===')
        self._test(r'\=\\\=\\=', r'\=\\\=\\', '')

    def test_no_unescaped_equal(self):
        for inp in '', 'xxx', r'\=', r'\\\=', r'\\\\\=\\\\\\\=\\\\\\\\\=':
            self._test(inp, inp, None)

    def test_no_split_in_variable(self):
        self._test(r'${a=b}', '${a=b}', None)
        self._test(r'=${a=b}', '', '${a=b}')
        self._test(r'${a=b}=', '${a=b}', '')
        self._test(r'\=${a=b}', r'\=${a=b}', None)
        self._test(r'${a=b}=${c=d}', '${a=b}', '${c=d}')
        self._test(r'${a=b}\=${c=d}', r'${a=b}\=${c=d}', None)
        self._test(r'${a=b}${c=d}${e=f}\=${g=h}=${i=j}',
                   r'${a=b}${c=d}${e=f}\=${g=h}', '${i=j}')

    def test_broken_variable(self):
        self._test('${foo=bar', '${foo', 'bar')

    def _test(self, inp, *exp):
        assert_equal(split_from_equals(inp), exp)


if __name__ == '__main__':
    unittest.main()



import signal
import time
import sys
import os.path


notify_path = sys.argv[1]

def log(msg, *extra_streams):
    for stream in (sys.stdout,) + extra_streams:
        stream.write(msg + '\n')
        stream.flush()

def ignorer(signum, frame):
    log('Ignoring signal %d.' % signum)

signal.signal(signal.SIGTERM, ignorer)
if hasattr(signal, 'SIGBREAK'):
    signal.signal(signal.SIGBREAK, ignorer)

with open(notify_path, 'w') as notify:
    log('Starting non-terminable process.', notify)


while True:
    try:
        time.sleep(0.1)
    except IOError:
        pass
    if not os.path.exists(notify_path):
        log('Stopping non-terminable process.')



import os
from pathlib import Path
import re
import subprocess
import sys


ROBOT_DIR = Path(__file__).parent.parent / 'src/robot'


def get_variables(path, name=None, version=None):
    return {'INTERPRETER': Interpreter(path, name, version)}


class Interpreter:

    def __init__(self, path, name=None, version=None):
        self.path = path
        self.interpreter = self._get_interpreter(path)
        if not name:
            name, version = self._get_name_and_version()
        self.name = name
        self.version = version
        self.version_info = tuple(int(item) for item in version.split('.'))

    def _get_interpreter(self, path):
        path = path.replace('/', os.sep)
        return [path] if os.path.exists(path) else path.split()

    def _get_name_and_version(self):
        try:
            output = subprocess.check_output(self.interpreter + ['-V'],
                                             stderr=subprocess.STDOUT,
                                             encoding='UTF-8')
        except (subprocess.CalledProcessError, FileNotFoundError) as err:
            raise ValueError('Failed to get interpreter version: %s' % err)
        name, version = output.split()[:2]
        name = name if 'PyPy' not in output else 'PyPy'
        version = re.match(r'\d+\.\d+\.\d+', version).group()
        return name, version

    @property
    def os(self):
        for condition, name in [(self.is_linux, 'Linux'),
                                (self.is_osx, 'OS X'),
                                (self.is_windows, 'Windows')]:
            if condition:
                return name
        return sys.platform

    @property
    def output_name(self):
        return '{i.name}-{i.version}-{i.os}'.format(i=self).replace(' ', '')

    @property
    def excludes(self):
        if self.is_pypy:
            yield 'require-lxml'
        for require in [(3, 7), (3, 8), (3, 9), (3, 10)]:
            if self.version_info < require:
                yield 'require-py%d.%d' % require
        if self.is_windows:
            yield 'no-windows'
        if not self.is_windows:
            yield 'require-windows'
        if self.is_osx:
            yield 'no-osx'
        if not self.is_linux:
            yield 'require-linux'

    @property
    def is_python(self):
        return self.name == 'Python'

    @property
    def is_pypy(self):
        return self.name == 'PyPy'

    @property
    def is_linux(self):
        return 'linux' in sys.platform

    @property
    def is_osx(self):
        return sys.platform == 'darwin'

    @property
    def is_windows(self):
        return os.name == 'nt'

    @property
    def runner(self):
        return self.interpreter + [str(ROBOT_DIR / 'run.py')]

    @property
    def rebot(self):
        return self.interpreter + [str(ROBOT_DIR / 'rebot.py')]

    @property
    def libdoc(self):
        return self.interpreter + [str(ROBOT_DIR / 'libdoc.py')]

    @property
    def testdoc(self):
        return self.interpreter + [str(ROBOT_DIR / 'testdoc.py')]

    def __str__(self):
        return f'{self.name} {self.version} on {self.os}'



from typing import Union


class Unknown:
    pass


def list_(argument: list[int], expected=None, same=False):
    _validate_type(argument, expected, same)


def list_with_unknown(argument: list[Unknown], expected=None):
    _validate_type(argument, expected)


def list_in_union_1(argument: Union[str, list[str]], expected=None, same=False):
    _validate_type(argument, expected, same)


def list_in_union_2(argument: Union[list[str], str], expected=None, same=False):
    _validate_type(argument, expected, same)


def tuple_(argument: tuple[int, bool, float], expected=None):
    _validate_type(argument, expected)


def tuple_with_unknown(argument: tuple[Unknown, int], expected=None):
    _validate_type(argument, expected)


def tuple_in_union_1(argument: Union[str, tuple[str, str, str]], expected=None):
    _validate_type(argument, expected)


def tuple_in_union_2(argument: Union[tuple[str, str, str], str], expected=None):
    _validate_type(argument, expected)


def homogenous_tuple(argument: tuple[int, ...], expected=None):
    _validate_type(argument, expected)


def homogenous_tuple_with_unknown(argument: tuple[Unknown, ...], expected=None):
    _validate_type(argument, expected)


def homogenous_tuple_in_union_1(argument: Union[str, tuple[str, ...]], expected=None):
    _validate_type(argument, expected)


def homogenous_tuple_in_union_2(argument: Union[tuple[str, ...], str], expected=None):
    _validate_type(argument, expected)


def dict_(argument: dict[int, float], expected=None, same=False):
    _validate_type(argument, expected, same)


def dict_with_unknown_key(argument: dict[Unknown, int], expected=None):
    _validate_type(argument, expected)


def dict_with_unknown_value(argument: dict[int, Unknown], expected=None):
    _validate_type(argument, expected)


def dict_in_union_1(argument: Union[str, dict[str, str]], expected=None, same=False):
    _validate_type(argument, expected, same)


def dict_in_union_2(argument: Union[dict[str, str], str], expected=None, same=False):
    _validate_type(argument, expected, same)


def set_(argument: set[bool], expected=None):
    _validate_type(argument, expected)


def set_with_unknown(argument: set[Unknown], expected=None):
    _validate_type(argument, expected)


def set_in_union_1(argument: Union[str, set[str]], expected=None):
    _validate_type(argument, expected)


def set_in_union_2(argument: Union[set[str], str], expected=None):
    _validate_type(argument, expected)


def nested_generics(argument: list[tuple[int, int]], expected=None, same=False):
    _validate_type(argument, expected, same)


def invalid_list(a: list[int, float]):
    pass


def invalid_tuple(a: tuple[int, float, ...]):
    pass


def invalid_dict(a: dict[int]):
    pass


def invalid_set(a: set[int, float]):
    pass


def _validate_type(argument, expected, same=False):
    if isinstance(expected, str):
        expected = eval(expected)
    if argument != expected or type(argument) != type(expected):
        atype = type(argument).__name__
        etype = type(expected).__name__
        raise AssertionError(f'{argument!r} ({atype}) != {expected!r} ({etype})')
    if same and argument is not expected:
        raise AssertionError(f'{argument} (id: {id(argument)}) is not same '
                             f'as {expected} (id: {id(expected)})')



class MyObject:

    def __init__(self):
        self.args = None

    def my_method(self, *args):
        if args == ('FAIL!',):
            raise RuntimeError('Expected failure')
        self.args = args

    def kwargs(self, arg1, arg2='default', **kwargs):
        kwargs = ['%s: %s' % item for item in sorted(kwargs.items())]
        return ', '.join([arg1, arg2] + kwargs)

    def __str__(self):
        return 'String presentation of MyObject'


obj = MyObject()



import base64
import unittest
import zlib

from robot.utils.compress import compress_text
from robot.utils.asserts import assert_equal, assert_true


class TestCompress(unittest.TestCase):

    def _test(self, text):
        compressed = compress_text(text)
        assert_true(isinstance(compressed, str))
        uncompressed = zlib.decompress(base64.b64decode(compressed)).decode('UTF-8')
        assert_equal(uncompressed, text)

    def test_empty_string(self):
        self._test('')

    def test_100_char_strings(self):
        self._test('100 Somewhat Random Chars ... als 13 asd 20a \n'
                   'Rsakjaf AdfSasda  asldjfaerew lasldjf awlkr aslk sd rl')

    def test_non_ascii(self):
        self._test('hyv\xe4')
        self._test('\u4e2d\u6587')


if __name__ == '__main__':
    unittest.main()



import builtins
import sys
import time
import traceback

from robot.utils import eq, normalize, timestr_to_secs

from objecttoreturn import ObjectToReturn


class ExampleLibrary:

    def print_(self, msg, stream='stdout'):
        """Print given message to selected stream (stdout or stderr)"""
        print(msg, file=getattr(sys, stream))

    def print_n_times(self, msg, count, delay=0):
        """Print given message n times"""
        for i in range(int(count)):
            print(msg)
            self._sleep(delay)

    def print_many(self, *msgs):
        """Print given messages"""
        for msg in msgs:
            print(msg, end=' ')
        print()

    def print_to_stdout_and_stderr(self, msg):
        print('stdout: ' + msg, file=sys.stdout)
        print('stderr: ' + msg, file=sys.stderr)

    def single_line_doc(self):
        """One line keyword documentation."""
        pass

    def multi_line_doc(self):
        """Only the first line of a multi line keyword doc should be logged.

        Thus for example this text here should not be there
        and neither should this.
        """
        pass

    def exception(self, name, msg="", class_only=False):
        exception = getattr(builtins, name)
        if class_only:
            raise exception
        raise exception(msg)

    def external_exception(self, name, msg):
        ObjectToReturn('failure').exception(name, msg)

    def implicitly_chained_exception(self):
        try:
            try:
                1/0
            except Exception:
                ooops
        except Exception:
            self._log_python_traceback()
            raise

    def explicitly_chained_exception(self):
        try:
            try:
                assert False
            except Exception as err:
                raise AssertionError('Expected error') from err
        except Exception:
            self._log_python_traceback()
            raise

    def _log_python_traceback(self):
        print(''.join(traceback.format_exception(*sys.exc_info())).rstrip())

    def return_string_from_library(self,string='This is a string from Library'):
        return string

    def return_list_from_library(self, *args):
        return list(args)

    def return_three_strings_from_library(self, one='one', two='two', three='three'):
        return one, two, three

    def return_object(self, name='<noname>'):
        return ObjectToReturn(name)

    def check_object_name(self, object, name):
        assert object.name == name, '%s != %s' % (object.name, name)

    def set_object_name(self, object, name):
        object.name = name

    def set_attribute(self, name, value):
        setattr(self, normalize(name), normalize(value))

    def get_attribute(self, name):
        return getattr(self, normalize(name))

    def check_attribute(self, name, expected):
        try:
            actual = getattr(self, normalize(name))
        except AttributeError:
            raise AssertionError("Attribute '%s' not set" % name)
        if not eq(actual, expected):
            raise AssertionError("Attribute '%s' was '%s', expected '%s'"
                                 % (name, actual, expected))

    def check_attribute_not_set(self, name):
        if hasattr(self, normalize(name)):
            raise AssertionError("Attribute '%s' should not be set" % name)

    def backslashes(self, count=1):
        return '\\' * int(count)

    def read_and_log_file(self, path, binary=False):
        mode = binary and 'rb' or 'r'
        _file = open(path, mode)
        print(_file.read())
        _file.close()

    def print_control_chars(self):
        print('\033[31mRED\033[m\033[32mGREEN\033[m')

    def long_message(self, line_length, line_count, chars='a'):
        line_length = int(line_length)
        line_count = int(line_count)
        msg = chars*line_length + '\n'
        print(msg*line_count)

    def loop_forever(self, no_print=False):
        i = 0
        while True:
            i += 1
            self._sleep(1)
            if not no_print:
                print('Looping forever: %d' % i)

    def write_to_file_after_sleeping(self, path, sec, msg=None):
        with open(path, 'w') as file:
            self._sleep(sec)
            file.write(msg or 'Slept %s seconds' % sec)

    def sleep_without_logging(self, timestr):
        seconds = timestr_to_secs(timestr)
        self._sleep(seconds)

    def _sleep(self, seconds):
        endtime = time.time() + float(seconds)
        while True:
            remaining = endtime - time.time()
            if remaining <= 0:
                break
            time.sleep(min(remaining, 0.001))

    def return_consumable_iterable(self, *values):
        return iter(values)

    def return_list_subclass(self, *values):
        return _MyList(values)

    def return_unrepresentable_objects(self, identifier=None, just_one=False):
        class FailingStr:

            def __init__(self, identifier=identifier):
                self.identifier = identifier

            def __str__(self):
                raise RuntimeError

        if just_one:
            return FailingStr()
        return FailingStr(), FailingStr()

    def fail_with_suppressed_exception_name(self, msg):
        raise MyException(msg)


class _MyList(list):
    pass


class MyException(AssertionError):
    ROBOT_SUPPRESS_NAME = True



from robot.libraries.BuiltIn import BuiltIn, register_run_keyword


class MyLibrary2:

    def keyword_only_in_library_2(self):
        print("Keyword from library 2")

    def keyword_in_both_libraries(self):
        print("Keyword from library 2")

    def keyword_in_all_resources_and_libraries(self):
        print("Keyword from library 2")

    def keyword_everywhere(self):
        print("Keyword from library 2")

    def keyword_in_tc_file_overrides_others(self):
        raise Exception("This keyword should not be called")

    def keyword_in_resource_overrides_libraries(self):
        raise Exception("This keyword should not be called")

    def no_operation(self):
        print("Overrides keyword from BuiltIn library")

    def replace_string(self):
        print("Overrides keyword from String library")
        return "I replace nothing!"

    def run_keyword_if(self, expression, name, *args):
        return BuiltIn().run_keyword_if(expression, name, *args)

register_run_keyword('MyLibrary2', 'run_keyword_if', 2)
